<?php
namespace Df\Payment;
use Df\Customer\Settings\BillingAddress;
use Df\Payment\Exception as DFPE;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Quote\Api\CartManagementInterface as IQM;
use Magento\Quote\Api\Data\CartInterface as IQuote;
use Magento\Quote\Model\Quote;
use Magento\Quote\Model\QuoteIdMask;
use Magento\Quote\Model\Quote\Payment as QP;
// 2016-07-18
final class PlaceOrderInternal {
	/**
	 * 2017-03-12
	 * @param int|string $cartId
	 * @param bool $isGuest
	 */
	private function __construct($cartId, $isGuest) {$this->_cartId = $cartId; $this->_isGuest = $isGuest;}

	/**
	 * 2016-07-18
	 * @return mixed|null
	 * @throws CouldNotSaveException
	 */
	private function _place() {
		/** @var mixed $result */
		try {
			BillingAddress::disable(!$this->s()->requireBillingAddress());
			try {
				/** @var int $orderId */
				$orderId = df_quote_m()->placeOrder($this->qid());
			}
			finally {
				BillingAddress::restore();
			}
			$result = df_order($orderId)->getPayment()->getAdditionalInformation(PlaceOrder::DATA);
		}
		catch (\Exception $e) {
			throw new CouldNotSaveException(__($this->message($e)), $e);
		}
		return $result;
	}
	
	/**
	 * 2016-07-18
	 * 2016-10-24
	 * Сообщение для покупателя функция возвращает,
	 * а сообщение для администратора — логирует.
	 * @used-by _place()
	 * @param \Exception|DFPE $e
	 * @return string
	 */
	private function message(\Exception $e) {
		/** @var bool $isSpecific */
		if (!($isSpecific = $e instanceof DFPE)) {
			$e = df_ef($e);
		}
		df_log($e);
		/** @var string $mc */
		/** @var string $md */
		list($mc, $md) =
			$isSpecific
			? [$e->messageC(), df_tag_if($e->messageD(), $e->isMessageHtml(), 'pre')]
			: [dfp_error_message(), df_etsd($e)]
		;
		return !$this->s()->test() ? $mc : df_cc_br($mc, __('Debug message:'), $md);
	}

	/**
	 * 2016-07-18
	 * @return int
	 */
	private function qid() {return dfc($this, function() {
		/** @var int|string $result */
		$result = $this->_cartId;
		/**
		 * 2016-07-18
		 * By analogy with https://github.com/magento/magento2/blob/2.1.0/app/code/Magento/Quote/Model/GuestCart/GuestCartManagement.php#L83-L87
		 */
		if ($this->_isGuest) {
			/** @var QuoteIdMask $quoteIdMask */
   			$quoteIdMask = df_load(QuoteIdMask::class, $result, true, 'masked_id');
			/** https://github.com/magento/magento2/blob/2.1.0/app/code/Magento/Quote/Setup/InstallSchema.php#L1549-L1557 */
			$result = $quoteIdMask['quote_id'];
			/** @var IQuote|Quote $quote */
			$quote = df_quote($result);
			$quote->setCheckoutMethod(IQM::METHOD_GUEST);
		}
		return $result;
	});}

	/**
	 * 2016-07-18
	 * @used-by _place()
	 * @used-by message()
	 * @return Settings
	 */
	private function s() {return dfc($this, function() {return
		dfp_method_by_p(df_quote($this->qid())->getPayment())->s()
	;});}

	/**
	 * 2017-03-12
	 * Для анонимных покупателей $cartId — это строка вида «63b25f081bfb8e4594725d8a58b012f7».
	 * https://github.com/CKOTech/checkout-magento2-plugin/issues/10
	 * @var int|string
	 */
	private $_cartId;

	/**
	 * 2017-03-12
	 * @var bool
	 */
	private $_isGuest;

	/**
	 * 2016-07-18
	 * @param int|string $cartId
	 * @param bool $isGuest
	 * @return mixed|null
	 * @throws CouldNotSaveException
	 */
	static function p($cartId, $isGuest) {return (new self($cartId, $isGuest))->_place();}
}